Public Class Form1

    '<<var
    ' The variable "StartImage" is used to save the index of the oldest image in the image ring buffer.
    Dim StartImage As Integer

    ' The variable "TimerCurrentImage" is used for the index of the currently displayed image,
    ' due auto repeat of the image sequence is running.    
    Dim TimerCurrentImage As Integer

    ' "Images" will contain a copy of the image ring buffer, in order to get faster acces to the images. 
    Dim Images() As TIS.Imaging.ImageBuffer


    ' The variable "FrameCount" is used to count the capture frames. If "FrameCount"
    ' is greater than the image ring buffer size, the "btnCapture" will be enabled and
    ' the user knows the image sequence in the ring buffer is now filled up completely.
    ' FrameCount will be set to 0 in btnLiveVideo_Click after the live video has been started.
    ' It is incremented in the ImageAvailable event handler.
    Dim FrameCount As Integer

    ' "Seconds" determins the length of the image ring buffer in seconds. 
    Dim Seconds As Integer
    '>>

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        IcImagingControl1.LiveCaptureLastImage = False
        IcImagingControl1.LiveCaptureContinuous = True
        Timer1.Enabled = False

        ' Since now video capture device has been selected yet, all buttons
        ' except the "Device" button are disabled.
        btnSettings.Enabled = False
        btnLiveVideo.Enabled = False
        btnCapture.Enabled = False
        sldImages.Enabled = False
        btnAutoRepeat.Enabled = False


        FrameCount = 0

        ' If the image sequence should contain more or less seconds, only the value of "Seconds" need to
        ' be changed here.
        Seconds = 2


        ' Resize the live display to the size of IC Imaging Control in order to display
        ' the complete live video.
        IcImagingControl1.LiveDisplayDefault = False
        IcImagingControl1.LiveDisplayHeight = IcImagingControl1.Height
        IcImagingControl1.LiveDisplayWidth = IcImagingControl1.Width
    End Sub

    ' btnDevice_Click()
    ' 
    ' Show device selection dialog in order to select a video capture device.
    _
    Private Sub btnDevice_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDevice.Click
        If IcImagingControl1.LiveVideoRunning Then
            IcImagingControl1.LiveStop()
        End If

        btnSettings.Enabled = False
        btnLiveVideo.Enabled = False
        btnCapture.Enabled = False

        '<<ringbuffersize
        ' Show device settings dialog.
        IcImagingControl1.ShowDeviceSettingsDialog()

        If IcImagingControl1.DeviceValid Then
            ' The last seconds before the capture button has been
            ' clicked should be saved in the image ring buffer. Thus
            ' the currently set frame rate multiplied by the value of "Seconds".
            IcImagingControl1.ImageRingBufferSize = Convert.ToInt32(IcImagingControl1.DeviceFrameRate * Seconds)
            '>>
            ' Now the range of the slider is set to the image buffers size:
            sldImages.Maximum = IcImagingControl1.ImageRingBufferSize - 1
            sldImages.Minimum = 0

            ' Show the number of images in the ring buffer.
            lblSequenceEnd.Text = IcImagingControl1.ImageRingBufferSize.ToString()

            btnSettings.Enabled = True
            btnLiveVideo.Enabled = True
        End If
    End Sub


    ' btnSettings_Click
    ' 
    ' Show the device settings dialog in order to adjust the video capture device's 
    ' properties. 
    '
    Private Sub btnSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSettings.Click
        If IcImagingControl1.DeviceValid Then
            ' Show device settings dialog.
            IcImagingControl1.ShowPropertyDialog()
        End If
    End Sub


    ' btnLiveVideo_Click
    ' 
    ' Start and stop the live video stream.
    '
    Private Sub btnLiveVideo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnLiveVideo.Click
        ' Start Live video.
        If IcImagingControl1.DeviceValid Then
            If IcImagingControl1.LiveVideoRunning Then
                IcImagingControl1.LiveStop()
                btnLiveVideo.Text = "Start Live"
            Else
                IcImagingControl1.LiveDisplay = True ' Show the live video.
                Timer1.Enabled = False
                sldImages.Enabled = False
                btnAutoRepeat.Enabled = False

                FrameCount = 0

                ' Disable the capture button. It will be enabled
                ' after a enough frames have been captured for
                ' a complete image sequence. This is indicated
                ' by the value of "FrameCount".
                btnCapture.Enabled = False

                IcImagingControl1.LiveStart()
                btnLiveVideo.Text = "Stop Live"
            End If
        End If
    End Sub


    ' btnCapture_Click
    ' 
    ' This is the event handler for the "Capture" button. If the "Capture" button
    ' has been clicked, the currently running live video is stopped. Then the oldest image
    ' is searched in the image ring buffer of IC Imaging Control. The oldest image
    ' is the first image to be displayed when the slider's position is set to 0. The
    ' index in the ring buffer of the oldest image is saved in the form global variable
    ' "StartImage".
    '
    Private Sub btnCapture_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCapture.Click
        If IcImagingControl1.DeviceValid Then
            ' Disable the "Capture" button after it has been clicked.
            btnCapture.Enabled = False

            '<<startcapture
            ' If the image sequence should contain images captures after the "Capture" 
            ' button has been clicked, a call to "Sleep()" should be inserted before
            ' "LiveStop()" is called, e.g.:
            '  System.Threading.Thread.Sleep(1000)
            ' This would the application continue capture one second after the "Capture"
            ' button has been clicked. 

            ' Now stop the live video stream.
            IcImagingControl1.LiveStop()
            btnLiveVideo.Text = "Start Live"
            ' Create a copy of the Imagesbuffers to get a faster access to the images.
            Images = IcImagingControl1.ImageBuffers
            '>>

            ' The following line is necessary to enable the display of the image buffers.
            ' in the ring buffer.

            IcImagingControl1.LiveDisplay = False

            '<<searcholdest
            ' Find the oldest image in the ring buffer. Its index is saved in
            ' the variable "StartImage".
            Dim i As Integer
            Dim MinStartTime As Double
            StartImage = 0
            MinStartTime = IcImagingControl1.ImageBuffers(StartImage).SampleStartTime
            For i = 1 To IcImagingControl1.ImageRingBufferSize - 1
                If Images(i).SampleStartTime < MinStartTime Then
                    StartImage = i
                    MinStartTime = Images(StartImage).SampleStartTime
                End If
            Next
            '>>
            sldImages.Enabled = True
            btnAutoRepeat.Enabled = True
            btnCapture.Enabled = False
        End If
    End Sub


    ' btnAutoRepeat_Click
    '
    ' Enables the auto repeat of the captured sequence.
    '
    Private Sub btnAutoRepeat_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAutoRepeat.Click
        If Not IcImagingControl1.LiveVideoRunning Then
            If Timer1.Enabled Then
                Timer1.Enabled = False
            Else
                Timer1.Interval = 10
                Timer1.Enabled = True
                TimerCurrentImage = 0
            End If
        End If
    End Sub

    ' sldImages_Scroll
    ' 
    ' The slider's position determines the image in the ring buffer to be
    ' displayed. 
    '
    Private Sub sldImages_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles sldImages.Scroll
        DisplayTheImage(sldImages.Value)
    End Sub

    ' timer1_Tick
    ' 
    ' If the automatically replay of the image sequence is in abled, the 
    ' images are iterated in this timer event handler.
    '
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        DisplayTheImage(TimerCurrentImage)
        TimerCurrentImage = TimerCurrentImage + 1
        If TimerCurrentImage >= IcImagingControl1.ImageRingBufferSize Then
            TimerCurrentImage = 0
        End If
        sldImages.Value = TimerCurrentImage
    End Sub

    ' DisplayTheImage
    ' 
    ' "DisplayTheImage" displayed the image in the ring buffer specified by
    ' the parameter "index". The specified "index" will be mapped to the sequence
    ' of images in the ring buffer. This means "index" is added to "StartIndex". If
    ' "Index" plus "StartIndex" is greater than the ImageRingBufferSize, then the
    ' new index in the ring buffer must start at 0. 
    '
    '<<display
    Private Sub DisplayTheImage(ByVal Index As Integer)
        Dim i As Integer
        With IcImagingControl1
            If Not .LiveVideoRunning Then
                i = StartImage + Index
                ' Handling a possible overflow, in case the i is greater then the ring buffer size.
                If i >= .ImageRingBufferSize Then
                    i = i - .ImageRingBufferSize
                End If
                .DisplayImageBuffer(Images(i))
            End If
        End With
    End Sub
    '>>

    ' EnableCaptureDelegate
    ' 
    ' This function is called from within "icImagingControl1_ImageAvailable()" as delegate.
    ' It is used for calling the private method "EnableCaptureButton()" that only enables the
    ' "Capture" button.
    '<<delegate
    Private Delegate Sub EnableCaptureDelegate()
    '>>

    ' EnableCaptureButton
    ' 
    ' This method is called by the "EnableCaptureDelegate()" from "icImagingControl1_ImageAvailable()"
    ' event handler after a sufficient number of frames have been captured into the image ring 
    ' buffer. It only enables the "Capture" button to inform the user that enough images for a new
    ' image sequence have been captured.

    '<<enablebutton
    Private Sub EnableCaptureButton()
        btnCapture.Enabled = True
    End Sub
    '>>


    ' icImagingControl1_ImageAvailable
    ' 
    ' This is the ImageAvailable event handler. The number of captured frames is counted by this method. 
    ' If the number of frames is higher than the ring buffer's size, the delegate "EnableCaptureDelegate"
    ' is called in order to enable the "Capture" button.
    '
    '<<imageavailable
    Private Sub IcImagingControl1_ImageAvailable(ByVal sender As System.Object, ByVal e As TIS.Imaging.ICImagingControl.ImageAvailableEventArgs) Handles IcImagingControl1.ImageAvailable
        If btnCapture.Enabled = False Then
            If FrameCount > IcImagingControl1.ImageRingBufferSize Then
                ' Now it is time to enable the capture button. 
                BeginInvoke(New EnableCaptureDelegate(AddressOf EnableCaptureButton))
            Else
                FrameCount = FrameCount + 1
            End If
        End If
    End Sub
    '>>
End Class
